'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (file, api, options) {
  var j = api.jscodeshift;

  var printOptions = options.printOptions || defaultPrintOptions;

  var root = j(file.source);
  var program = root.get().value.program;
  var body = program.body;


  var classes = [];

  function makeClass(className, type, defaultValues) {
    var classNameIdentifier = j.identifier(className);
    var staticMethods = [(0, _fromJS2.default)(j, className, defaultValues, type.properties)];
    var instanceMethods = type.properties.reduce(function (methods, prop) {
      methods.push((0, _getter2.default)(j, prop), (0, _setter2.default)(j, prop));
      return methods;
    }, [(0, _toJS2.default)(j, className, type.properties)]);

    var classDeclaration = j.exportNamedDeclaration(j.classDeclaration(classNameIdentifier, j.classBody(staticMethods.concat(instanceMethods)), j.identifier('ImmutableModel')));
    var comments = [' /////////////////////////////////////////////////////////////////////////////', '', ' NOTE: THIS CLASS IS GENERATED. DO NOT MAKE CHANGES HERE.', '', ' If you need to update this class, update the corresponding flow type above', ' and re-run the flow-immutable-models codemod', '', ' /////////////////////////////////////////////////////////////////////////////'];
    classDeclaration.comments = comments.map(function (comment) {
      return j.commentLine(comment);
    });
    return [classDeclaration];
  }

  function parseType(td) {
    var typeDef = Object.assign({}, td);
    delete typeDef.start;
    delete typeDef.end;
    delete typeDef.loc;
    delete typeDef.extra;

    if (typeDef.id) {
      typeDef.id = parseType(typeDef.id);
    }
    if (typeDef.key) {
      typeDef.key = parseType(typeDef.key);
    }
    if (typeDef.value) {
      typeDef.value = parseType(typeDef.value);
    }

    if (typeDef.types) {
      typeDef.types = typeDef.types.map(parseType);
    }
    if (typeDef.properties) {
      typeDef.properties = typeDef.properties.map(parseType);
    }

    return typeDef;
  }

  root.find(j.ExportNamedDeclaration).filter(function (p) {
    if (p.node.exportKind === 'type') {
      var identifier = p.node.declaration.id.name;
      return (0, _withoutModelTypeSuffix.endsWithModelType)(identifier);
    }
    return false;
  }).forEach(function (p) {
    var identifier = p.node.declaration.id.name;
    var className = (0, _withoutModelTypeSuffix.withoutModelTypeSuffix)(identifier);
    var parsed = parseType(p.node.declaration.right);
    var defaultValuesName = 'default' + (0, _capitalize2.default)(className) + 'Values';
    var defaultValues = root.find(j.VariableDeclaration).filter(function (path) {
      return path.node.declarations.some(function (dec) {
        return dec.id.name === defaultValuesName;
      });
    });

    classes.push({
      className: className,
      classDef: makeClass(className, parsed, defaultValues.size() === 1 ? defaultValuesName : null)
    });
  });

  classes.forEach(function (_ref) {
    var className = _ref.className;
    var classDef = _ref.classDef;

    var alreadyExportedClass = root.find(j.ExportNamedDeclaration).filter(function (path) {
      return path.node.declaration.type === 'ClassDeclaration' && path.node.declaration.id.name === className;
    });

    if (alreadyExportedClass.size() === 1) {
      alreadyExportedClass.replaceWith(classDef);
    } else {
      body.push.apply(body, _toConsumableArray(classDef));
    }
  });

  return root.toSource(printOptions);
};

var _capitalize = require('./helpers/capitalize');

var _capitalize2 = _interopRequireDefault(_capitalize);

var _fromJS = require('./helpers/fromJS');

var _fromJS2 = _interopRequireDefault(_fromJS);

var _getter = require('./helpers/getter');

var _getter2 = _interopRequireDefault(_getter);

var _setter = require('./helpers/setter');

var _setter2 = _interopRequireDefault(_setter);

var _toJS = require('./helpers/toJS');

var _toJS2 = _interopRequireDefault(_toJS);

var _withoutModelTypeSuffix = require('./helpers/withoutModelTypeSuffix');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var defaultPrintOptions = { quote: 'single', trailingComma: true };